use_radix_sort	= 1
use_bucket_sort	= 0


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; reset the actor list
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!macro actor_initialize {
		ldx #actor_last-actor_first
		ldy #actor_last
		sty actor_free
.loop		dey
		dex
		sty actor_link+actor_first+1,x
		bne .loop

		stx actor_count

	;;	ldx #actor_sentinel
		dex
		stx actor_link+actor_first
		stx actor_head
		stx actor_dying

		ldx #a_sentinel_class
		stx actor_sentinel_class
		}


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; use this macro to append any dying actors to the free list.
;; an intermediate 'death' list is needed since we don't want them to respawn
;; too early and be confused for the original during collision detection
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!macro actor_reclaim_the_dead {
		lda actor_dying
		bmi .empty

.loop		tax
		lda actor_link,x
		bpl .loop

		ldy actor_free
		sty actor_link,x
		ldy actor_dying
		sty actor_free
		sta actor_dying

.empty		}


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; add a linked set of actors to the death queue
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!macro actor_collect_death_list .head {
		lax .head
		bmi .empty

.loop		dec actor_count
		ldy actor_link,x
		bmi .break
		dec actor_count
		ldx actor_link,y
		bpl .loop

		ldx actor_dying
		stx actor_link,y
		jmp .join

.break		ldy actor_dying
		sty actor_link,x

.join		sta actor_dying

.empty		}


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; allocate a new actor at the top of the list (and thus screen).
;; the number is returned in x. y is preserved
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!macro actor_spawn .failure {
		ldx actor_free
	;;	bpl *+3
	;;	jam
		bmi .failure

		lda actor_link,x
		sta actor_free

		lda actor_head
		sta actor_link,x
		stx actor_head

		inc actor_count
		}


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; allocate a new actor and place it directly after it's parent.
;; parent is specified in x and the child is returned in y
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!macro actor_spawn_at .failure {
		ldy actor_free
	;;	bpl *+3
	;;	jam
		bmi .failure

		lda actor_link,y
		sta actor_free

		lda actor_link,x
		sta actor_link,y
		sty actor_link,x

		inc actor_count
		}


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; adds the actor in x to the list
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!macro actor_spawn_specific {
		lda actor_head
		sta actor_link,x
		stx actor_head

		inc actor_count
		}


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; run through the actor list and invoke each one's update function.
;; a linear copy of the original actor list is also preserved on the stack for
;; future collision detection
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
actor_update	!zone
		tsx			;; save a copy of the actor list on a
		stx .original_stack	;; separate stack frame
		ldx #<actor_order_stack	;; +1
		txs

		ldx #actor_head-actor_link

actor_update_continue
		+common ~actor_update_continue_cb

		stx actor_previous

		lda actor_link,x
.continue	tax
		pha

		lda actor_class,x
		sta .branch
		+bne_rel ~.branch,.end

.start
		;; actor class 0 is used to identify live players
a_player_class	= *-.start
		jmp a_player_update
a_player_bullet_class = *-.start
		jmp a_player_bullet_update
a_player_laser_class = *-.start
		jmp a_player_laser_update
a_buzzer_class	= *-.start
		jmp a_buzzer_update
a_streaker_class = *-.start
		jmp a_streaker_update
a_seeker_class	= *-.start
		jmp a_seeker_update
a_sweeper_class	= *-.start
		jmp a_sweeper_update
a_vertical_bullet_class = *-.start
		jmp a_vertical_bullet_update
a_directional_bullet_class = *-.start
		jmp a_directional_bullet_update
a_explosion_class = *-.start
		jmp a_explosion_update
a_chain_explosion_class = *-.start
		jmp a_chain_explosion_update
a_zombie_class	= *-.start
		+jmp ~actor_update_continue_cb
a_sentinel_class = *-.start		;; must be the last class, because of
					;; the collision table

.end		+ldx_imm ~.original_stack
		txs
		rts

		;; move the actor being updated from the main actor list to
		;; the death queue
actor_update_die +common ~actor_update_die_cb
		dec actor_count

		ldy actor_previous
		lda actor_link,x
		sta actor_link,y

		cpx #actor_first	;; don't add player's to the usual list
		bcc .dying_player

		ldy actor_dying
		sty actor_link,x
		stx actor_dying
		jmp .continue

.dying_player	ldy #a_zombie_class
		sty actor_class,x
		jmp .continue


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; a couple of debugging macros to verify certain actor list invariants
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!macro dbg_check_actor_count {
		anc #$00

		ldx actor_head
		bmi .last
.loop		adc #1
		ldy actor_link,x
		bmi .last
		adc #1
		ldx actor_link,y
		bpl .loop

.last		cmp actor_count
		beq .ok
		jam
.ok		}

		!macro dbg_check_actor_total {
		anc #$00

		ldx actor_head
.main_loop	bmi .last_main
		cpx #actor_first
		adc #0
		ldy actor_link,x
		bmi .last_main
		cpy #actor_first
		adc #0
		ldx actor_link,y
		bpl .main_loop

.last_main	ldx actor_free
		bmi .last_free
.free_loop	adc #1
		ldy actor_link,x
		bmi .last_free
		adc #1
		ldx actor_link,y
		bpl .free_loop

.last_free	ldx actor_dying
		bmi .last_dying
.dying_loop	adc #1
		ldy actor_link,x
		bmi .last_dying
		adc #1
		ldx actor_link,y
		bpl .dying_loop

.last_dying	cmp #actor_last-actor_first+1
		beq .ok
		jam
.ok		}

		!macro dbg_is_actor_list_sorted {
		lda #$00
		sta r0

		ldx actor_head
		bmi .end

.loop		lda actor_ypos,x
		cmp r0
		bcs .ok
		jam
.ok		sta r0

		lda actor_link,x
		tax
		bpl .loop

.end		}


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; bucket sort
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!if use_bucket_sort {
actor_bucket	= zscratch
actor_bucket_end = zscratch+40

actor_sort	!zone
		ldx actor_head
		bpl .not_empty
		rts

		;; clear the table first
.not_empty	ldx #$ff
		lda #$fe
		stx actor_bucket+40	;; offscreen bucket
	;;	stx actor_bucket+39	;; don't care
		!for i,0,19 {
		stx actor_bucket+38-i*2
		sax actor_bucket+37-i*2
		sbx #$01
		}
		stx actor_bucket+38-i*2

		;; partition the sprites into a set of buckets based on their
		;; y coordinates
		ldx actor_head
.partition	ldy actor_ypos,x
		lda actor_bucket_index,y
		tay

		lda actor_bucket,y
		stx actor_bucket,y

		ldy actor_link,x
		sta actor_link,x
		bmi .continue

		ldx actor_ypos,y
		lda actor_bucket_index,x
		tax

		lda actor_bucket,x
		sty actor_bucket,x

		ldx actor_link,y
		sta actor_link,y
		bpl .partition

		;; kill off any invisible actors
.continue	+actor_collect_death_list actor_bucket+40
		rts

		;; a "weighted" table used divide y coordinates into one of 40
		;; buckets
		+align
actor_bucket_index = *
		!fill 34,40		;; bucket 0 is "invisible"
		!for i,0,202 {
		!byte (i*40)/202	;; 5 lines/bucket
		}
		!fill <-*,40
		}


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; radix sort
;;
;; [ bucket sort ]
;; bucket init:	               168 cycles
;; sort:       3 + 28.5 x 32 = 912 cycles
;; mux writer:      ~16 x 32 = 512 cycles
;; total:        25 rasters / 1592 cycles
;;
;; [ radix sort ]
;; bucket init:                101 cycles
;; lsd sort:    37 + 24 x 32 = 805 cycles
;; msd sort:    88 + 28 x 32 = 984 cycles
;; mux writer:       ~8 x 32 = 256 cycles
;; total:	 34 rasters / 2146 cycles
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!if use_radix_sort {
actor_sort	= actor_radix_sort

actor_radix_lsd	= zscratch+0		;; 16 bytes
actor_radix_msd	= zscratch+16		;; 14 bytes

actor_bucket	= actor_radix_msd	;; expose these for the multiplexer
actor_bucket_end = actor_radix_msd+13

actor_radix_sort !zone

		;; initialize bucket lists
		ldx #$ff
		!for i,0,16 {
		stx actor_radix_lsd+i
		}

		stx actor_radix_msd+13	;; offscreen bucket
	;;	stx actor_radix_msd+12	;; don't care

		lda #$fe
		stx actor_radix_msd+11	;; -1
		sax actor_radix_msd+10	;; -2
		sbx #$01 
		stx actor_radix_msd+9	;; -3
		sax actor_radix_msd+8	;; -4
		sbx #$01 
		stx actor_radix_msd+7	;; -5
		sax actor_radix_msd+6	;; -6
		sbx #$01 
		stx actor_radix_msd+5	;; -7
		sax actor_radix_msd+4	;; -8
		sbx #$01
		stx actor_radix_msd+3	;; -9
		sax actor_radix_msd+2	;; -10
		sbx #$01 
		stx actor_radix_msd+1	;; -11
		sax actor_radix_msd+0	;; -12


		;; lsd sort
		!macro actor_radix_lsd_iteration_x {
		lda actor_ypos,x
		and #$0f
		tay

		lda actor_radix_lsd,y
		stx actor_radix_lsd,y
		ldy actor_link,x
		sta actor_link,x
		}

		!macro actor_radix_lsd_iteration_y {
		lda actor_ypos,y
		and #$0f
		tax

		lda actor_radix_lsd,x
		sty actor_radix_lsd,x
		ldx actor_link,y
		sta actor_link,y
		}

		lda actor_count
		bne .lsd_continue
		rts
.lsd_continue	and #$07
		tax
		lda .lsd_jmptab,x
		sta .lsd_branch

		lax actor_head
		tay
		+bpl_rel ~.lsd_branch

.lsd_jmptab	!byte .lsd_x1-.lsd_jmptab
		!byte .lsd_y8-.lsd_jmptab
		!byte .lsd_x7-.lsd_jmptab
		!byte .lsd_y6-.lsd_jmptab
		!byte .lsd_x5-.lsd_jmptab
		!byte .lsd_y4-.lsd_jmptab
		!byte .lsd_x3-.lsd_jmptab
		!byte .lsd_y2-.lsd_jmptab

.lsd_x1		+actor_radix_lsd_iteration_x
.lsd_y2		+actor_radix_lsd_iteration_y
.lsd_x3		+actor_radix_lsd_iteration_x
.lsd_y4		+actor_radix_lsd_iteration_y
.lsd_x5		+actor_radix_lsd_iteration_x
.lsd_y6		+actor_radix_lsd_iteration_y
.lsd_x7		+actor_radix_lsd_iteration_x
.lsd_y8		+actor_radix_lsd_iteration_y
		bpl .lsd_x1

		;; msd sort
		!for i,16 {
		ldx actor_radix_lsd+16-i
		bmi +

	-	ldy actor_ypos,x
		lda actor_bucket_index,y
		tay

		lda actor_radix_msd,y
		stx actor_radix_msd,y
		ldy actor_link,x
		sta actor_link,x
		bmi +

		ldx actor_ypos,y
		lda actor_bucket_index,x
		tax

		lda actor_radix_msd,x
		sty actor_radix_msd,x
		ldx actor_link,y
		sta actor_link,y
		bpl -

	+	}


		;; kill off any invisible actors
		+actor_collect_death_list actor_bucket_end
		rts


		;; a table used divide actor y coordinates into the
		;; most-significant radix buckets. the last bucket serves to
		;; collect off-screen actors
		+align
actor_bucket_index
		!fill 34,13		;; upper border
		!for i,<*,202 {		;; -.
		!byte (i>>4)-2		;;  |-- visible
		}			;; -'
		!fill <-*,13		;; lower border
		}