		;; a shared field (a base class if you will) between the
		;; player and its bullets, used to identify the responsible
		;; player in a collision. it also needs to be preserved in
		;; collisions inheriting the player's actor
a_player_owner = actor_user1

a_player_reload	= actor_user2
a_player_anim_dir = actor_user3
a_player_anim_frame = actor_user4
a_player_invulnerable = actor_user5
a_player_entering = actor_user6
a_player_battery = actor_user7		;; laser energy left

					;; battery state
a_player_charging = actor_user8		;; $00 = loading,
					;; $01 = full,
					;; $80 = discharging

a_player_xvel	= 1
a_player_yvel	= 2

a_player_xmin	= 13
a_player_xmax	= 158
a_player_ymin	= $37
a_player_ymax	= $d8

a_player_initial_lives = 3
a_player_force_laser = 0


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; reset the player's at the start of a game
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!macro a_player_initialize {
		ldx #a_player_2nd-a_player_1st
.player		lda #a_zombie_class
		sta actor_class+a_player_1st,x
		lda #$00
		sta a_player_score_lo+a_player_1st,x
		sta a_player_score_md+a_player_1st,x
		sta a_player_score_hi+a_player_1st,x
		sta a_player_battery+a_player_1st,x
		sta a_player_charging+a_player_1st,x
		dex
		bpl .player
		}


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; figure out the x-coordinate of the leftmost and rightmost player for
;; enemy tracking
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!macro a_player_update_partition {
		ldy actor_class+a_player_1st
		beq .1st_alive
		lax actor_xpos+a_player_2nd
		ldy actor_class+a_player_2nd
		beq .write
		lda #(a_player_xmin+a_player_xmax)*1/3
		ldx #(a_player_xmin+a_player_xmax)*2/3
		bne .write			;; bra

.1st_alive	lax actor_xpos+a_player_1st
		ldy actor_class+a_player_2nd
		bne .write
		ldx actor_xpos+a_player_2nd
		cmp actor_xpos+a_player_2nd
		bcc .write
		tax
		lda actor_xpos+a_player_2nd

.write		sta a_player_partition+0
		stx a_player_partition+1
		}


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; x = player actor #
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!macro a_player_spawn .failure {
		lda a_player_lives,x
		beq .failure
		bpl .dont_check_input

		lda cia1_pra-a_player_1st,x
		and #%10000
		bne .failure

		lda #a_player_initial_lives
		sta a_player_lives,x

.dont_check_input
		dec a_player_lives,x

		;;;;;;;;;;;;;;;;

spawn

		+actor_spawn_specific

		lda a_player_initial_xpos,x
		sta actor_xpos,x
		lda #a_player_ymax+20
		sta actor_ypos,x

		lda #$ff
		sta actor_collision,x
		sta a_player_invulnerable,x
		sta a_player_anim_frame,x
		lda #$04
		sta a_player_anim_dir,x
		lda #$0a
		sta a_player_entering,x

		lda #a_player_class
		sta actor_class,x

		sta a_player_reload,x
		sta a_player_battery,x
		sta a_player_charging,x

		txa
		sta a_player_owner,x
		}

a_player_initial_xpos = *-a_player_1st
		!byte (a_player_xmin+a_player_xmax)*1/3
		!byte (a_player_xmin+a_player_xmax)*2/3

a_player_cached_input = *-a_player_1st
		!byte $ff,$ff


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; animate the player's movements
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!zone
a_player_pattern_tab = *

.c		= *-a_player_pattern_tab
		!byte spr_player_center
		!byte spr_player_center
		!byte spr_player_center
		!byte spr_player_center

.u		= *-a_player_pattern_tab
		!byte spr_player_center
		!byte spr_player_up+0
		!byte spr_player_up+0
		!byte spr_player_up+1

.d		= *-a_player_pattern_tab
		!byte spr_player_center
		!byte spr_player_down+0
		!byte spr_player_down+0
		!byte spr_player_down+1

.l		= *-a_player_pattern_tab
		!byte spr_player_center
		!byte spr_player_left+0
		!byte spr_player_left+0
		!byte spr_player_left+1

.r		= *-a_player_pattern_tab
		!byte spr_player_center
		!byte spr_player_right+0
		!byte spr_player_right+0
		!byte spr_player_right+1

a_player_direction_tab = *
		!byte .c		;; 1111 up+down+left+right
		!byte .d		;; 1110 down+left+right
		!byte .u		;; 1101 up+left+right
		!byte .c		;; 1100 left+right
		!byte .r		;; 1011 up+down+right
		!byte .r		;; 1010 down+right
		!byte .r		;; 1001 up+right
		!byte .r		;; 1000 right
		!byte .l		;; 0111 up+down+left
		!byte .l		;; 0110 down+left
		!byte .l		;; 0101 up+left
		!byte .l		;; 0100 left
		!byte .c		;; 0011 up+down
		!byte .d		;; 0010 down
		!byte .u		;; 0001 up
		!byte .c		;; 0000 centered

		!macro a_player_animate .input {
.increase	= 40
.decrease	= 40

		lda .input
		and #%1111
		tay
		lda a_player_direction_tab,y
		beq .no_direction
direction	cmp a_player_anim_dir,x
		bne .no_direction

		lda a_player_anim_frame,x
		adc #.increase
		bcs .keep_frame
		bcc .update_frame

.no_direction	tay
		lda a_player_anim_frame,x
		sbc #.decrease
		bcs .update_frame
		tya
		sta a_player_anim_dir,x
		bcc .keep_frame

.update_frame	sta a_player_anim_frame,x

		;;
.keep_frame	lda a_player_anim_frame,x
		anc #%11000000
		rol
		asl
		adc a_player_anim_dir,x

		tay
		lda a_player_pattern_tab,y
		sta actor_pattern,x
		}


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; a bit of firing logic
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!macro a_player_fire .input {
		lsr a_player_reload,x
		bne .dont_fire
		lsr .input
		bcc .fire

		lda a_player_charging,x	;; don't stop discharging until the
		bmi .fire		;; battery runs out

					;; start discharging the laser when
		beq .dont_fire		;; the battery is full and the player
		ror a_player_charging,x	;; releases the fire button

		;; common initialization
.fire		+actor_spawn_at .dont_fire
		txa
		sta a_player_owner,y

		;; spawn the right kind of bullet
		lda a_player_charging,x
		bmi .laser

		!if a_player_force_laser {
		+a_player_laser_initialize
		lda #a_player_laser_reload
		} else {
		+a_player_bullet_initialize
		lda #a_player_bullet_reload
		}
		bne .reload		;; bra

.laser		dec a_player_battery,x
		bne .not_empty
		asl a_player_charging,x

.not_empty	+a_player_laser_initialize
		lda #a_player_laser_reload

.reload		sta a_player_reload,x
.dont_fire	}


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; perform most of the player update in a separate pass.
;; this is necessary to allow the laser beam sprites to lock on to the
;; player's updated position
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!zone
.return		rts

a_player_early_update
		;; do we need to respawn the player?
		lda actor_class,x
		beq .alive
		cmp #a_zombie_class
		bne .return

		+a_player_spawn .return

		;; read joystick input
.input		= r0

.alive		lda cia1_pra-a_player_1st,x
		ldy a_player_entering,x
		beq .enterance_complete
		dec a_player_entering,x
		lda #%11111110
.enterance_complete
		sta .input

		;; animation
		+a_player_animate .input

		;; color animation
		ldy .player_color,x

		lda a_player_invulnerable,x
		beq .vulnerable
		dec a_player_invulnerable,x
		and #$10
		beq .vulnerable
		ldy #vicclr_white+$80

.vulnerable	sty actor_attrib,x

		;; vertical movement
		lda actor_ypos,x

		lsr .input
		bcs .not_up
		adc #-a_player_yvel
		cmp #a_player_ymin
		bcs .not_up
		lda #a_player_ymin

.not_up		lsr .input
		bcs .not_down
		adc #a_player_yvel
		cmp #a_player_ymax
		bcc .not_down
		lda #a_player_ymax

.not_down	sta actor_ypos,x

		;; horizontal movement
		lda actor_xpos,x

		lsr .input
		bcs .not_left
		adc #-a_player_xvel
		cmp #a_player_xmin
		bcs .not_left
		lda #a_player_xmin

.not_left	lsr .input
		bcs .not_right
		adc #a_player_xvel
		cmp #a_player_xmax
		bcc .not_right
		lda #a_player_xmax

.not_right	sta actor_xpos,x

		;; fire!
		+a_player_fire .input
		rts

.player_color	= *-a_player_1st
		!byte vicclr_cyan+$80
		!byte vicclr_lred+$80


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; most of the work is done in a_player_early_update.
;; just do a bit of collision detection here
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!zone
a_player_update lda #game_countdown_reset;; at least one player is alive
		sta game_countdown

		ldy actor_collision,x
		bmi .no_collision
		lda a_player_invulnerable,x
		bne .non_fatal

		+a_chain_explosion_replace

.non_fatal	lda #$ff
		sta actor_collision,x

.no_collision	+jmp ~actor_update_continue_cb


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; a = the score to be awarded
;; y = player actor (the bullet's owner)
;;
;; x and y are preserved
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
player_award_collision_score !zone
		stx .save_x
		tax

		sed

		clc
		lda player_scores+0,x
		adc a_player_score_lo,y
		sta a_player_score_lo,y
		lda player_scores+1,x
		adc a_player_score_md,y
		sta a_player_score_md,y
		lda player_scores+2,x
		adc a_player_score_hi,y
		sta a_player_score_hi,y

		cld

		;; also take the chance to recharge the battries
		lda a_player_charging,y
		bne .dont_recharge
		lda player_scores+3,x
	;;	clc
		adc a_player_battery,y
		bcc .still_charging
		lda #$01
		sta a_player_charging,y	;; battery full
		lda #$ff
.still_charging	sta a_player_battery,y

.dont_recharge	+ldx_imm ~.save_x
		rts

player_scores	= *

player_score_low = *-player_scores
		!24 $25
		!byte 1
player_score_medium = *-player_scores
		!24 $50
		!byte 2
player_score_high = *-player_scores
		!24 $100
		!byte 3


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; y = the next active player, or clear the z flag if both are dead
;; x is preserved
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
a_player_target	!zone
		+lda_imm ~.target,a_player_1st
		eor #a_player_1st XOR a_player_2nd
		tay
		lda actor_class,y
		bne .try_next
		sty .target
		rts

.try_next	ldy .target
		lda actor_class,y
		rts