;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; the actual (indirect) nmi vector
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!zone
		+align 4
dc_nmi		jmp (cia1_ta_hi)

		;; handle charset switching within the first odd (non-doubled)
		;; row of a scrolled screen. note that this solution will
		;; occasionally jitter by one cycle, but solving it properly
		;; would involve another raster irq or rewriting parts of
		;; nmi2/3 before and after their first execution
dc_early_nmi	sta .save_a+1
		nop
		lda #<dc_nmi
		lsr vic_ram
		sta cpu_nmi+0
.save_a		lda #$ff
		jmp cia2_rti


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!zone
.cia_detect	sta cia_model
		jmp cia1_rti
.dummy_nmi	jmp cia2_rti


		;;;;;;;; basic interrupt hardware initialization ;;;;;;;;

dc_initialize	lda #%01111111		;; clear interrupt masks
		sta cia1_icr
		sta cia2_icr
		lda #%00000000
		sta vic_irqmask

		sta cia1_cra		;; stop timers
		sta cia1_crb
		sta cia2_cra
		sta cia2_crb

		bit cia1_icr		;; acknowledge pending interrupts
		bit cia2_icr
		lsr vic_irq

		sta cia1_ta_hi		;; reset timer high-bytes
		sta cia1_tb_hi
		sta cia2_ta_hi
		sta cia2_tb_hi

		lda #$40		;; an alternative way of acknowledging
		sta cia1_rti		;; and returning from cia interrupts
		sta cia2_rti

cia1_rti	= cia1_sdr
cia2_rti	= cia2_sdr

		lda <.dummy_nmi		;; set nmi vector (in case of restore
		sta cpu_nmi+0		;; interrupts)
		lda >.dummy_nmi
		sta cpu_nmi+1


		;;;;;;;; detect cia model ;;;;;;;;

.delay		bit vic_controlv	;; make sure we're not disturbed by
		bpl .delay		;; any badlines or sprite cycles

		lda #<.cia_detect
		sta cpu_irq+0
		lda #>.cia_detect
		sta cpu_irq+1

		lda #2			;; trigger a one-shot interrupt almost
		sta cia1_ta_lo		;; immediately
		lda #%10000001
		sta cia1_icr
		lda #%00011001
		sta cia1_cra
		cli

		nop
		lda #1			;; the timing is determined by whether
		lda #0			;; or not we we reach the second line

		lda #%01111111		;; and stop cia 1a from generating any
		sta cia1_icr		;; further interrupts


		;;;;;;;; install the interrupt handlers ;;;;;;;;

		ldx #.total_size-1	;; copy the code into place
.outer		ldy .size_tab+7

.inner		lda .nmi0,x
		sta $0700,y
		dex
		dey
		bpl .inner

		dec .outer+1
		dec .inner+5
		bpl .outer

		lda #<dc_nmi		;; set nmi vector
		sta cpu_nmi+0
		lda #>dc_nmi
		sta cpu_nmi+1

		lda #%10000011		;; enable interrupts for both of cia2's
		sta cia2_icr		;; timers
		rts


		;;;;;;;; NMI interrupt handlers ;;;;;;;;

		;; min: { 28,27,33 } - 42 = 46 (x 12 = 552)
		;; max: { 29,28,35 } - 40 = 52 (x 12 = 624)

.ram		= vic_ram
.controlv	= vic_controlv

		;; this should only ever be generated by restore interrupts
.nmi0		!byte $82,$00		;; nop #$?? / memory config 5 (ram+io)
		jmp cia2_rti
dc_nmi0_size	= *-.nmi0

.nmi1		jmp cia2_rti		;; @ $0100
dc_nmi1_size	= *-.nmi1

		;; switch to charset #1, restore y-scroll
.nmi2		nop			;; @ $0200
		lsr .ram
		inc .controlv
		jmp cia2_rti
dc_nmi2_size	= *-.nmi2

.nmi3		inc .controlv		;; @ $0300
		lsr .ram
		jmp cia2_rti
dc_nmi3_size	= *-.nmi3

		;; line double
.nmi4		dec .controlv		;; @ $0400
		jmp cia2_rti
dc_nmi4_size	= *-.nmi4

.nmi5		bit cia2_icr		;; @ $0500
		dec .controlv
		rti
dc_nmi5_size	= *-.nmi5

		;; switch to charset #2
.nmi6		nop			;; @ $0600
		asl .ram
		jmp cia2_rti
dc_nmi6_size	= *-.nmi6

.nmi7		bit cia2_icr		;; @ $0700
		asl .ram
		rti
dc_nmi7_size	= *-.nmi7

.total_size	= *-.nmi0

		;; a table used for installing the handlers
		+single_page 8
.size_tab	!byte dc_nmi0_size-1
		!byte dc_nmi1_size-1
		!byte dc_nmi2_size-1
		!byte dc_nmi3_size-1
		!byte dc_nmi4_size-1
		!byte dc_nmi5_size-1
		!byte dc_nmi6_size-1
		!byte dc_nmi7_size-1