;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!macro game_run {

		;;;;;;;; reset game logic ;;;;;;;;

		lda #-1
		sta game_ready
		lda #game_active_run
		sta game_active
		lda #raster_active_run
		sta raster_active

		lda #$00
		sta view_ypos
		sta game_frame
	;;	sta game_frame_odd

		lda #game_countdown_reset
		sta game_countdown

		lda #<level_start
		sta script+0
		lda #>level_start
		sta script+1

		+actor_initialize
		+a_player_initialize

		jsr actor_update
		jsr actor_sort
		jsr mux_write

		jsr map_reset
		jsr statusbar_clear


		;;;;;;;; static vic settings ;;;;;;;;

		ldx #%00000000
		stx vic_sdx
		stx vic_sdy
		stx vic_sback
		dex
		stx vic_smc

		lda #vicclr_black
		sta vic_sc11
		lda #vicclr_white
		sta vic_sc01

	;;	lda #vicclr_black
	;;	sta vic_cborder

		lda level_palette+0
		sta vic_cbg0
		lda level_palette+1
		sta vic_cbg1
		lda level_palette+2
		sta vic_cbg2


		;;;;;;;; begin fade in ;;;;;;;;

		jsr fade_lock
		ldx #-8
		stx game_fade


		;;;;;;;; prepare display interrupts ;;;;;;;;

		lda #<bottom_irq	;; trigger main raster irq
		sta cpu_irq+0
		lda #>bottom_irq
		sta cpu_irq+1
		lda #bottom_line
		sta vic_line
		lsr vic_irq
		lda #%00000001
		sta vic_irqmask

		lda #%00010100		;; set vic bank
		sta cia2_pra


		;;;;;;;; the main loop ;;;;;;;;

idle		;; 13 cycles per iteration
		;; 312 * 63 / 13 = 1512 ($5e8)

.hi_loop	inc game_idle_hi
.lo_loop	inc game_idle_lo
		beq .hi_loop
		+delay 3

		;; to cleanly exit the game loop we first have to shut down
		;; the raster interrupts, controlled by the raster_active
		;; flag which in turn resets game_active
game_active	jmp .lo_loop
game_active_run = opc_jmp_abs
game_active_break = opc_nop_abs
		}


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; run one (logical) frame's worth of the game
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
game_logic	!zone

		!zone {

		;; fade in:  -8 .. -1
		;; fade out: +8 .. +1

		+lda_imm ~game_fade
		beq .not_fading
		bmi .fade_in

		sec			;; fade out
		sbc #1
		bne .not_done
		ldx #raster_active_break
		stx raster_active
.not_done	sta game_fade
		jmp fade_update

.fade_in	and #%111		;; fade in
		inc game_fade
		jmp fade_update

.not_fading	lda #8
		bit view_ypos
		beq .dont
		dec game_countdown
		bne .dont
		sta game_fade

		jmp fade_lock

.dont		}

		;;;;;;;;

		inc game_frame
		lda game_frame
		and #$01
		sta game_frame_odd

		jsr collision_detection

		ldx #a_player_1st
		jsr a_player_early_update
		inx
		jsr a_player_early_update
		+a_player_update_partition

		jsr script_execute

		jsr map_scroll

		jsr actor_update

		+actor_reclaim_the_dead

		jsr statusbar_redraw

		lda #$00
		sta game_idle_lo
		sta game_idle_hi

		jmp actor_sort