;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; various utility macros
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!macro delay .cycles {
		!do while .cycles >= 4 {
		nop
		!set .cycles = .cycles - 2
		}

		!if .cycles <= 1 { !serious "cannot wait a single cycle" }
		!if .cycles = 2 { nop }
		!if .cycles = 3 { !byte $04,$00 }
		}

		;; align and keep track of the number of bytes lost
		!set align_loss = 0

		!macro align .size {
.start		!align .size-1,0
		!set align_loss = align_loss+(*-.start)
		}

		!macro align { +align 256 }

		;; keep a block of memory from crossing a page boundary
		!macro single_page .size {
		!if >* != >(*+.size) {
		+align
		}
		}

		!macro sprite_line .pattern {
		!byte ^.pattern,>.pattern,<.pattern
		}

		!macro random {
		lda rand_lo
		lsr
		rol rand_hi
		bcc .skip
		eor #$b4
.skip		sta rand_lo
		eor rand_hi
		}

		;; skip one or two bytes without affecting the flags
		!macro nop8 { !byte opc_nop_imm }
		!macro nop16 { !byte opc_nop_abs }


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; utility macros for placing labels at an opcode's parameter, to facilitate
;; self-modifying code
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!macro lda_imm ~.label,.value {
		!byte opc_lda_imm
.label		!byte .value
		}

		!macro ldx_imm ~.label,.value {
		!byte opc_ldx_imm
.label		!byte .value
		}

		!macro ldy_imm ~.label,.value {
		!byte opc_ldy_imm
.label		!byte .value
		}

		!macro cmp_imm ~.label,.value {
		!byte opc_cmp_imm
.label		!byte .value
		}

		!macro cpx_imm ~.label,.value {
		!byte opc_cpx_imm
.label		!byte .value
		}

		!macro cpy_imm ~.label,.value {
		!byte opc_cpy_imm
.label		!byte .value
		}

		!macro bne_rel ~.label,.value {
.label		= *+1
		bne .value
		}

		!macro beq_rel ~.label,.value {
.label		= *+1
		beq .value
		}

		!macro bpl_rel ~.label,.value {
.label		= *+1
		bpl .value
		}

		!macro bmi_rel ~.label,.value {
.label		= *+1
		bmi .value
		}

		!macro bcc_rel ~.label,.value {
.label		= *+1
		bcc .value
		}

		!macro bcs_rel ~.label,.value {
.label		= *+1
		bcs .value
		}

		!macro bvc_rel ~.label,.value {
.label		= *+1
		bvc .value
		}

		!macro bvs_rel ~.label,.value {
.label		= *+1
		bvs .value
		}

		!macro lda_imm ~.l { +lda_imm ~.l,0 }
		!macro ldx_imm ~.l { +ldx_imm ~.l,0 }
		!macro ldy_imm ~.l { +ldy_imm ~.l,0 }
		!macro cmp_imm ~.l { +cmp_imm ~.l,0 }
		!macro cpx_imm ~.l { +cpx_imm ~.l,0 }
		!macro cpy_imm ~.l { +cpy_imm ~.l,0 }
		!macro bne_rel ~.l { +bne_rel ~.l,*+2 }
		!macro beq_rel ~.l { +beq_rel ~.l,*+2 }
		!macro bpl_rel ~.l { +bpl_rel ~.l,*+2 }
		!macro bmi_rel ~.l { +bmi_rel ~.l,*+2 }
		!macro bcc_rel ~.l { +bcc_rel ~.l,*+2 }
		!macro bcs_rel ~.l { +bcs_rel ~.l,*+2 }
		!macro bvc_rel ~.l { +bvc_rel ~.l,*+2 }
		!macro bvs_rel ~.l { +bvs_rel ~.l,*+2 }