		;; assembler configuration
		!to "main.prg",cbm
		!cpu 6510
		!ct raw
		!vicell "ll"

		;; standard acme includes
		!source "stdinc/std.a"
		!source "stdinc/vic.a"
		!source "stdinc/cia1.a"
		!source "stdinc/cia2.a"

		;; basic header
		*= $0801
		!byte $0b,$08
		!word 2007
		!byte $9e
		!text "2061"
		!byte 0,0,0
		jmp start


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; various data
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
sprites		!binary "data/sprites.spr"
sprites_size	= *-sprites


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!source "src/macros.inc"
		!source "src/opcodes.inc"
		!source "src/common_branch.inc"
		!source "src/debug.inc"
		!source "src/doublechar.asm" ;; needs to be included early
		!source "src/memory.inc"

		!source "src/math.asm"
		!source "src/math_tab.asm"

		!source "src/map.asm"
		!source "src/panel.asm"
		!source "src/raster.asm"
		!source "src/fade.asm"
		!source "src/actor.asm"
		!source "src/multiplexer.asm"
		!source "src/collision.asm"
		!source "src/script.asm"

		!source "src/actors/explosion.asm"
		!source "src/actors/chain_explosion.asm"
		!source "src/actors/vertical_bullet.asm"
		!source "src/actors/directional_bullet.asm"
		!source "src/actors/buzzer.asm"
		!source "src/actors/streaker.asm"
		!source "src/actors/seeker.asm"
		!source "src/actors/sweeper.asm"
		!source "src/actors/player_bullet.asm"
		!source "src/actors/player_laser.asm"
		!source "src/actors/player.asm"

		!source "src/game.asm"
		!source "src/statusbar.asm"

		+align
fade_tab	!binary "data/fadetab.bin"


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; the main function
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
start		!zone
		lda #%01111111		;; stop timers from generating any
		sta cia1_icr		;; interrupts
		sta cia2_icr

		sei
		cld
		ldx #<main_stack
		txs

.seed		= $973a			;; reset RNG
		lda #<.seed
		sta rand_lo
		lda #>.seed
		sta rand_hi


		;;;;;;;; load the graphics to the VIC bank ;;;;;;;;

		lda #%00110011		;; switch out the io page
		sta $01

charset		= level_tileset

		ldx #0			;; install the charset
.copy_cs	lda charset+$000,x : sta vic_charset1+$000,x
		lda charset+$100,x : sta vic_charset1+$100,x
		lda charset+$200,x : sta vic_charset1+$200,x
		lda charset+$300,x : sta vic_charset1+$300,x
		lda charset+$400,x : sta vic_charset1+$400,x
		lda charset+$500,x : sta vic_charset1+$500,x
		lda charset+$600,x : sta vic_charset1+$600,x
		lda charset+$700,x : sta vic_charset1+$700,x
		lda charset+$800,x : sta vic_charset2+$000,x
		lda charset+$900,x : sta vic_charset2+$100,x
		lda charset+$a00,x : sta vic_charset2+$200,x
		lda charset+$b00,x : sta vic_charset2+$300,x
		lda charset+$c00,x : sta vic_charset2+$400,x
		lda charset+$d00,x : sta vic_charset2+$500,x
		lda charset+$e00,x : sta vic_charset2+$600,x
		lda charset+$f00,x : sta vic_charset2+$700,x
		inx
		bne .copy_cs

.copy_spr	!for i,0,>(sprites_size+$ff) {
		lda sprites+i*$100,x
		sta vic_sprites+i*$100,x
		}
		inx
		bne .copy_spr

		lda #%011		;; switch out all rom and bring back
		sta $00			;; the io
		lda #%01
		sta $01

		jsr dc_initialize


		;;;;;;;; run the game ;;;;;;;;

.blank		ldx #0			;; blank the screen
		stx vic_controlv
		stx vic_cborder

		dex
		stx a_player_lives+a_player_1st
		stx a_player_lives+a_player_2nd

.outer		ldx #1
.inner		lda cia1_pra,x
		and #%10000
		beq .join
		dex
		bpl .inner
		bmi .outer

.join		lda #a_player_initial_lives
		sta a_player_lives+a_player_1st,x

		+game_run
		jmp .blank


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; level data
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!source "levels/snow_script.asm"
level_tileset	= *+$0000
level_palette	= *+$1000
level_map	= *+$1003
		!binary "levels/snow_map.bin"
level_map_end	= *
