;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; video matrix layout
;;
;; 8+ 0+
;; -- --
;; 00 00
;; 00 01
;; 02 01
;; 02 03
;; 04 03
;; 04 05
;; 06 05
;; 06 07
;; 08 07
;; 08 09
;; 10 09
;; 10 11
;; 12 11
;; 12 13
;; 14 13
;; 14 15
;; 16 15
;; 16 17
;; 18 17
;; 18 19
;; 20 19
;; 20 21
;; 22 21
;; 22 23
;;[24 23]
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; the actual video matrix/color ram shifting is distributed (fairly) evenly
;; across all 16 pixel positions. note that the first line is visible in both
;; (odd/even) buffers thus refilling it is timing sensitive
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!zone
.frame_length	!byte  0, 3, 2, 3, 2, 3, 2, 3
		!byte  2, 3, 2, 3, 2, 3, 2, 3
		!byte  2, 3, 2, 3, 2, 3, 2, 2
		!byte  2, 3, 2, 3, 2, 3, 2, 2

.frame_offset	!byte  0,40,43,45,48,50,53,55
		!byte 58,60,63,65,68,70,73,75
		!byte  0, 2, 5, 7,10,12,15,17
		!byte 19,21,24,26,29,31,34,36

;;.frame_length	!byte  0, 6, 5, 6, 5, 6, 5, 5
;;		!byte  5, 5, 5, 4, 5, 5, 5, 4
;;
;;.frame_offset	!byte  0,40,46,51,57,62,68,73
;;		!byte  0, 5,10,15,19,24,29,34

map_scroll	+lda_imm ~map_ypos
		tax
		clc
		adc #1
		and #31
		sta map_ypos
		lsr
		sta view_ypos

		ldy .frame_length,x
		+beq .refill
		lda .frame_offset,x
		tax

.copy		!for i,0,12 {
		lda vic_screen+80*i,x
		sta vic_screen+80*i+40,x
		lda vic_color+80*i,x
		sta vic_color+80*i+40,x
		}
		inx
		dey
		+bne .copy
		rts

.refill		ldy #0

		!for i,0,40 {
		lda (map_ptr),y
		sta vic_screen+i
		iny
		lda (map_ptr),y
		sta vic_color+i
		iny
		}

		lax map_ptr+0
		sbx #-80
		stx map_ptr+0
		bcc .no_overflow
		inc map_ptr+1

.no_overflow	lda map_ptr+1
		cpx #<level_map_end-80
		sbc #>level_map_end-80
		bmi .no_rewind

		lda #<level_map
		sta map_ptr+0
		lda #>level_map
		sta map_ptr+1

.no_rewind	rts


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
map_reset	!zone
		lda #<level_map
		sta map_ptr+0
		lda #>level_map
		sta map_ptr+1

		lda #<40*24
		sta .color+1
		sta .screen+1
		lda #>vic_color+40*24
		sta .color+2
		lda #>vic_screen+40*24
		sta .screen+2

		lda #16
		sta map_ypos
		lsr
		sta view_ypos

.yloop		ldx #40-1
		ldy #80-1

.xloop		lda (map_ptr),y
.color		sta vic_color+40*24,x
		dey
		lda (map_ptr),y
.screen		sta vic_screen+40*24,x
		dey
		dex
		bpl .xloop

		lax map_ptr+0
		sbx #-80
		stx map_ptr+0
		bcc .no_l_overflow
		inc map_ptr+1

.no_l_overflow	lax .color+1
		sbx #80
		stx .color+1
		bcs .no_c_overflow
		dec .color+2

.no_c_overflow	lax .screen+1
		sbx #80
		stx .screen+1
		bcs .yloop
		lda #(>vic_screen)-1
		dcp .screen+2
		bne .yloop
		rts