;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; helper macros
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!set _alloc_ptr = $a000
		!set _zalloc_ptr = $05	;; $00-$04 = 1st nmi handler

		!if dc_nmi0_size != _zalloc_ptr { !error }

		!macro alloc ~.label,.size {
		!set .label = _alloc_ptr
		!set _alloc_ptr = _alloc_ptr+.size
		}

		!macro alloc ~.label {
		+alloc ~.label,1
		}

		!macro zalloc ~.label,.size {
		!set .label = _zalloc_ptr
		!set _zalloc_ptr = _zalloc_ptr+.size
		!if _zalloc_ptr > $100 { "error" }
		}

		!macro zalloc ~.label {
		+zalloc ~.label,1
		}


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; actor attributes
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
actor_limit	= 32			;; max 32 actors

		+zalloc ~actor_sentinel_class
		+zalloc ~actor_class,32
		+zalloc ~actor_link,32
		+zalloc ~actor_xpos,32
		+zalloc ~actor_ypos,32
		+zalloc ~actor_attrib,32

		+alloc ~actor_pattern,32
		+alloc ~actor_collision,32
		+alloc ~actor_user1,32
		+alloc ~actor_user2,32
		+alloc ~actor_user3,32
		+alloc ~actor_user4,32
		+alloc ~actor_user5,32
		+alloc ~actor_user6,32
		+alloc ~actor_user7,32
		+alloc ~actor_user8,32

		+zalloc ~actor_count
		+zalloc ~actor_head
		+zalloc ~actor_previous
		+zalloc ~actor_free
		+zalloc ~actor_dying

actor_sentinel	= -1
a_player_1st	= 0			;; the first two actors are the players
a_player_2nd	= 1
actor_first	= 2			;; the first general-purpose actor
actor_last	= 31


		;;;;;;;; player-specific state ;;;;;;;;

		;; bcd encoded score
		+alloc ~a_player_score,6
a_player_score_lo = a_player_score+0
a_player_score_md = a_player_score+2
a_player_score_hi = a_player_score+4

		;; negative: not yet joined
		+alloc ~a_player_lives,2

		+alloc ~a_player_partition,2


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; miscellaneous variables
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		;; zeropage workspace
		+zalloc ~zscratch,41	;; we only need 30 bytes when using
					;; the radix sort

r0		= zscratch+0
r1		= zscratch+1
r2		= zscratch+2
r3		= zscratch+3
r4		= zscratch+4
r5		= zscratch+5
r6		= zscratch+6
r7		= zscratch+7
r8		= zscratch+8
r9		= zscratch+9
r10		= zscratch+10
r11		= zscratch+11
r12		= zscratch+12
r13		= zscratch+13
r14		= zscratch+14
r15		= zscratch+15

		;;
		+zalloc ~irq_save_a
		+zalloc ~irq_save_x

		+zalloc ~rand_lo
		+zalloc ~rand_hi

		+zalloc ~map_ptr,2

		+zalloc ~mux_m7

		+zalloc ~view_ypos

		+zalloc ~game_frame
		+zalloc ~game_frame_odd
		+zalloc ~game_idle_lo
		+zalloc ~game_idle_hi
		+alloc ~game_ready
		+alloc ~game_countdown

game_countdown_reset = $ff

		+zalloc ~script,2
script_lo	= script+0
script_hi	= script+1

		+alloc ~panel_line,1
		+alloc ~panel_irq,2

		;; fading buffers
		+alloc ~fade_vic_buf,fade_vic_palette_size
		+alloc ~fade_actor_buf,actor_limit
		+alloc ~fade_map_buf,38
		+alloc ~fade_status_buf,39


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; stack page usage
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
nmi_stack_sentinel = $100+dc_nmi1_size	;; the first few bytes are used as an
					;; nmi interrupt handler

actor_order_stack = $13f		;; 32 actors plus some space for actor
					;; updates and interrupt frames

main_stack	= $1ff


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; vic bank allocations
;;
;; [ lost sprites ]
;; video matrix: 16
;; charsets:     64
;; panel matrix:  2
;; panel bitmap:  5
;; irq vectors:   1
;; blank sprite:  1
;; total:        89 (167 left)
;;
;; 28/168 sprites used (16.67%, 1/6th)
;; 27/175 sprites used (15.43%)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
vic_screen	= $c000
vic_sprpat	= $c3f8
vic_charset1	= $c800
vic_charset2	= $d800
vic_color	= $d800
vic_sprites	= $e000

vic_panel_matrix = $cf98
vic_panel_sprpat = $cff8
vic_panel_bitmap = $fcc0

		!set _spr_ptr = <(vic_sprites>>6)

		!macro spr_alloc ~.label,.count {
		!set .label = _spr_ptr
		!set _spr_ptr = _spr_ptr+.count
		}

		!macro spr_alloc ~.label {
		+spr_alloc ~.label,1
		}

		+spr_alloc ~spr_blank

		+spr_alloc ~spr_buzzer
		+spr_alloc ~spr_streaker
		+spr_alloc ~spr_sweeper

		+spr_alloc ~spr_player_center
		+spr_alloc ~spr_player_right,2
		+spr_alloc ~spr_player_left,2
		+spr_alloc ~spr_player_up,2
		+spr_alloc ~spr_player_down,2

		+spr_alloc ~spr_explosion,10

		+spr_alloc ~spr_player_bullet,1
		+spr_alloc ~spr_enemy_bullet,1
		+spr_alloc ~spr_laser,4