mux_spacing	= 22


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; kick the multiplexer into action by programming the first eight sprites and
;; triggering IRQs for the rest (if necessary)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!macro mux_register ~.src,.dst {
		+lda_imm ~.src
		sta .dst
		}

mux_display	+mux_register ~mux_x0,vic_xs0
		+mux_register ~mux_x1,vic_xs1
		+mux_register ~mux_x2,vic_xs2
		+mux_register ~mux_x3,vic_xs3
		+mux_register ~mux_x4,vic_xs4
		+mux_register ~mux_x5,vic_xs5
		+mux_register ~mux_x6,vic_xs6
		+mux_register ~mux_x7,vic_xs7

		+mux_register ~mux_y0,vic_ys0
		+mux_register ~mux_y1,vic_ys1
		+mux_register ~mux_y2,vic_ys2
		+mux_register ~mux_y3,vic_ys3
		+mux_register ~mux_y4,vic_ys4
		+mux_register ~mux_y5,vic_ys5
		+mux_register ~mux_y6,vic_ys6
		+mux_register ~mux_y7,vic_ys7

		+mux_register ~mux_p0,vic_sprpat+0
		+mux_register ~mux_p1,vic_sprpat+1
		+mux_register ~mux_p2,vic_sprpat+2
		+mux_register ~mux_p3,vic_sprpat+3
		+mux_register ~mux_p4,vic_sprpat+4
		+mux_register ~mux_p5,vic_sprpat+5
		+mux_register ~mux_p6,vic_sprpat+6
		+mux_register ~mux_p7,vic_sprpat+7

		+mux_register ~mux_c0,vic_cs0
		+mux_register ~mux_c1,vic_cs1
		+mux_register ~mux_c2,vic_cs2
		+mux_register ~mux_c3,vic_cs3
		+mux_register ~mux_c4,vic_cs4
		+mux_register ~mux_c5,vic_cs5
		+mux_register ~mux_c6,vic_cs6
		+mux_register ~mux_c7,vic_cs7

		lda mux_m7
		sta vic_msb_xs

		+mux_register ~mux_active,vic_sactive

		bit vic_ss_collided	;; flush the collision register

		;; start the irq chain
		+lda_imm ~mux_l0
		beq .skip

		sta vic_line
		lda #<mux_first_irq
		sta cpu_irq+0
		lda #>mux_first_irq
		sta cpu_irq+1
		rts

.skip		lda panel_line
		sta vic_line
		lda panel_irq+0
		sta cpu_irq+0
		lda panel_irq+1
		sta cpu_irq+1
		rts


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; interrupt handlers for reprogramming the final 24 sprites
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
mux_irq_abort_opc = opc_lda_imm

		!macro mux_irq i,~.x,~.y,~.p,~.c,~.m,~.l {
.sprite		= i%8

.this_irq	sta irq_save_a

		lda vic_ss_collided
		sta mux_cmask+mux_cmask_pitch*(i-8)

		+mux_register ~.x,vic_xs0+.sprite*2
		+mux_register ~.c,vic_cs0+.sprite
		+mux_register ~.p,vic_sprpat+.sprite
		+mux_register ~.m,vic_msb_xs
		+mux_register ~.y,vic_ys0+.sprite*2

		asl vic_irq

		!if i < 31 {
		!set .next_irq = *+25

		!if (>.this_irq) < (>.next_irq) {
		!set .next_irq = .next_irq+3
		inc cpu_irq+1
		}

		+lda_imm ~.l
		beq .abort
		sta vic_line
		cmp vic_line
		bcc .dont_wait		;; raster <= line
		beq .dont_wait

		lda #<.next_irq
		sta cpu_irq+0

		lda irq_save_a
		rti

.abort		jmp mux_irq_abort

		!if .next_irq != * {
		!serious "incorrect code length"
		}

.dont_wait	= *+2			;; skip initial 'sta irq_save_a'
		}
		}

		!zone
mux_first_irq	= *

.irq8		+mux_irq 8, ~mux_x8, ~mux_y8, ~mux_p8, ~mux_c8, ~mux_m8 ,~mux_l1
.irq9		+mux_irq 9, ~mux_x9, ~mux_y9, ~mux_p9, ~mux_c9, ~mux_m9 ,~mux_l2
.irq10		+mux_irq 10,~mux_x10,~mux_y10,~mux_p10,~mux_c10,~mux_m10,~mux_l3
.irq11		+mux_irq 11,~mux_x11,~mux_y11,~mux_p11,~mux_c11,~mux_m11,~mux_l4
.irq12		+mux_irq 12,~mux_x12,~mux_y12,~mux_p12,~mux_c12,~mux_m12,~mux_l5
.irq13		+mux_irq 13,~mux_x13,~mux_y13,~mux_p13,~mux_c13,~mux_m13,~mux_l6
.irq14		+mux_irq 14,~mux_x14,~mux_y14,~mux_p14,~mux_c14,~mux_m14,~mux_l7
.irq15		+mux_irq 15,~mux_x15,~mux_y15,~mux_p15,~mux_c15,~mux_m15,~mux_l8
.irq16		+mux_irq 16,~mux_x16,~mux_y16,~mux_p16,~mux_c16,~mux_m16,~mux_l9
.irq17		+mux_irq 17,~mux_x17,~mux_y17,~mux_p17,~mux_c17,~mux_m17,~mux_l10
.irq18		+mux_irq 18,~mux_x18,~mux_y18,~mux_p18,~mux_c18,~mux_m18,~mux_l11
.irq19		+mux_irq 19,~mux_x19,~mux_y19,~mux_p19,~mux_c19,~mux_m19,~mux_l12
.irq20		+mux_irq 20,~mux_x20,~mux_y20,~mux_p20,~mux_c20,~mux_m20,~mux_l13
.irq21		+mux_irq 21,~mux_x21,~mux_y21,~mux_p21,~mux_c21,~mux_m21,~mux_l14
.irq22		+mux_irq 22,~mux_x22,~mux_y22,~mux_p22,~mux_c22,~mux_m22,~mux_l15
.irq23		+mux_irq 23,~mux_x23,~mux_y23,~mux_p23,~mux_c23,~mux_m23,~mux_l16
.irq24		+mux_irq 24,~mux_x24,~mux_y24,~mux_p24,~mux_c24,~mux_m24,~mux_l17
.irq25		+mux_irq 25,~mux_x25,~mux_y25,~mux_p25,~mux_c25,~mux_m25,~mux_l18
.irq26		+mux_irq 26,~mux_x26,~mux_y26,~mux_p26,~mux_c26,~mux_m26,~mux_l19
.irq27		+mux_irq 27,~mux_x27,~mux_y27,~mux_p27,~mux_c27,~mux_m27,~mux_l20
.irq28		+mux_irq 28,~mux_x28,~mux_y28,~mux_p28,~mux_c28,~mux_m28,~mux_l21
.irq29		+mux_irq 29,~mux_x29,~mux_y29,~mux_p29,~mux_c29,~mux_m29,~mux_l22
.irq30		+mux_irq 30,~mux_x30,~mux_y30,~mux_p30,~mux_c30,~mux_m30,~mux_l23
.irq31		+mux_irq 31,~mux_x31,~mux_y31,~mux_p31,~mux_c31,~mux_m31,~.dummy

		;; fallthrough

mux_irq_abort	!zone
		lda panel_line
		sta vic_line
		cmp vic_line
		bcc .dont_wait
		beq .dont_wait

		lda panel_irq+0
		sta cpu_irq+0
		lda panel_irq+1
		sta cpu_irq+1
		lda irq_save_a
		rti

.dont_wait	lda irq_save_a
		jmp (panel_irq)


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; extract the sprite parameters from a sorted actors
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
mux_writer_continue_opc = opc_lax_zpy
mux_writer_break_opc = opc_rts_imp


		;;;;;;;; find the next non-empty bucket ;;;;;;;;

		!macro mux_writer_link {
		lax actor_link,y
		bpl .continue

		lda actor_bucket_end,x
		bpl .bucket_found
.bucket_loop	tax
		lda actor_bucket_end,x
		bmi .bucket_loop
.bucket_found	sta actor_link,y	;; chain together a complete actor list

.continue	tay
		}


		;;;;;;;; write the basic attributes into some smc loads ;;;;;;;;

		!macro mux_writer .x,.y,.p,.c,.l {
		lax actor_ypos,y
		sta .y

		!if .l {
		sbx #-mux_spacing
		stx .l
		}

		lda actor_pattern,y
		sta .p

		lda actor_attrib,y
		sta .c
		asl

		lda actor_xpos,y
		rol
		sta .x
		}


		;;;;;;;; update msb-x for for the main set of sprites ;;;;;;;;

		!macro mux_writer_msbx .bit,.old,.new {
.mask		= 1<<.bit

		lda .old
		and #<NOT .mask

		!if .mask = 1 {
		adc #$00
		} else {
		bcc .clear
		ora #.mask
.clear		}

		sta .new
		}


		;;;;;;;; handle a specific number of visible sprites ;;;;;;;;

		!macro mux_writer_enter i,.break,.line {
		!if i < 31 {		;; break out of the loop early
		lda #mux_writer_break_opc
		sta .break
		}

		jsr mux_writer_begin

		!if i < 31 {
		lda #mux_writer_continue_opc
		sta .break
		}

		;; when less than eight sprites are visible we need to adjust
		;; the msb byte and set the sprite-active bitfield
		!if i < 7 {
.shift		= 7-i

		!if .shift > 2 {
		lda mux_m7
		!for j,.shift { lsr }
		sta mux_m7
		} else {
		!for j,.shift { lsr mux_m7 }
		}

		lda #(2<<i)-1
		sta mux_active
		}

		;; terminate the actor list
		ldx #actor_sentinel
		stx actor_link,y

		;; as well as the irq chain
		!if i < 31 {
	;;	ldx #$00
		inx
		stx .line
		}
		rts
		}


		;;;;;;;; ? ;;;;;;;;

		!zone
.empty		ldx #$00
		stx mux_active
		stx mux_l0

	;;	ldx #actor_sentinel
		dex
		stx actor_head
		rts

mux_write	lda actor_count
		beq .empty
		asl
		adc actor_count
		sta .branch

		ldy #<actor_bucket-actor_bucket_end
.bucket_loop	lax actor_bucket_end,y
		bpl .bucket_found
		ldy actor_bucket_end,x
		bmi .bucket_loop
		tya
.bucket_found	sta actor_head
.continue	tay

		+bpl_rel ~.branch

		jmp .empty		;; unused

		jmp .enter0  : jmp .enter1  : jmp .enter2  : jmp .enter3
		jmp .enter4  : jmp .enter5  : jmp .enter6  : jmp .enter7
		jmp .enter8  : jmp .enter9  : jmp .enter10 : jmp .enter11
		jmp .enter12 : jmp .enter13 : jmp .enter14 : jmp .enter15
		jmp .enter16 : jmp .enter17 : jmp .enter18 : jmp .enter19
		jmp .enter20 : jmp .enter21 : jmp .enter22 : jmp .enter23
		jmp .enter24 : jmp .enter25 : jmp .enter26 : jmp .enter27
		jmp .enter28 : jmp .enter29 : jmp .enter30;: jmp .enter31

		+mux_writer_enter 31,0,0

.enter0		+mux_writer_enter 0, .break0, mux_l0
.enter1		+mux_writer_enter 1, .break1, mux_l0
.enter2		+mux_writer_enter 2, .break2, mux_l0
.enter3		+mux_writer_enter 3, .break3, mux_l0
.enter4		+mux_writer_enter 4, .break4, mux_l0
.enter5		+mux_writer_enter 5, .break5, mux_l0
.enter6		+mux_writer_enter 6, .break6, mux_l0
.enter7		+mux_writer_enter 7, .break7, mux_l0
.enter8		+mux_writer_enter 8, .break8, mux_l1
.enter9		+mux_writer_enter 9, .break9, mux_l2
.enter10	+mux_writer_enter 10,.break10,mux_l3
.enter11	+mux_writer_enter 11,.break11,mux_l4
.enter12	+mux_writer_enter 12,.break12,mux_l5
.enter13	+mux_writer_enter 13,.break13,mux_l6
.enter14	+mux_writer_enter 14,.break14,mux_l7
.enter15	+mux_writer_enter 15,.break15,mux_l8
.enter16	+mux_writer_enter 16,.break16,mux_l9
.enter17	+mux_writer_enter 17,.break17,mux_l10
.enter18	+mux_writer_enter 18,.break18,mux_l11
.enter19	+mux_writer_enter 19,.break19,mux_l12
.enter20	+mux_writer_enter 20,.break20,mux_l13
.enter21	+mux_writer_enter 21,.break21,mux_l14
.enter22	+mux_writer_enter 22,.break22,mux_l15
.enter23	+mux_writer_enter 23,.break23,mux_l16
.enter24	+mux_writer_enter 24,.break24,mux_l17
.enter25	+mux_writer_enter 25,.break25,mux_l18
.enter26	+mux_writer_enter 26,.break26,mux_l19
.enter27	+mux_writer_enter 27,.break27,mux_l20
.enter28	+mux_writer_enter 28,.break28,mux_l21
.enter29	+mux_writer_enter 29,.break29,mux_l22
.enter30	+mux_writer_enter 30,.break30,mux_l23


		;;;;;;;; 0-7 ;;;;;;;;

mux_writer_begin

		+mux_writer mux_x0,mux_y0,mux_p0,mux_c0,mux_l0
		ror mux_m7
.break0		+mux_writer_link

		+mux_writer mux_x1,mux_y1,mux_p1,mux_c1,mux_l1
		ror mux_m7
.break1		+mux_writer_link

		+mux_writer mux_x2,mux_y2,mux_p2,mux_c2,mux_l2
		ror mux_m7
.break2		+mux_writer_link

		+mux_writer mux_x3,mux_y3,mux_p3,mux_c3,mux_l3
		ror mux_m7
.break3		+mux_writer_link

		+mux_writer mux_x4,mux_y4,mux_p4,mux_c4,mux_l4
		ror mux_m7
.break4		+mux_writer_link

		+mux_writer mux_x5,mux_y5,mux_p5,mux_c5,mux_l5
		ror mux_m7
.break5		+mux_writer_link

		+mux_writer mux_x6,mux_y6,mux_p6,mux_c6,mux_l6
		ror mux_m7
.break6		+mux_writer_link

		lda #%11111111
		sta mux_active

		+mux_writer mux_x7,mux_y7,mux_p7,mux_c7,mux_l7
		ror mux_m7
.break7		+mux_writer_link


		;;;;;;;; 8-15 ;;;;;;;;

		+mux_writer mux_x8,mux_y8,mux_p8,mux_c8,mux_l8
		+mux_writer_msbx 0,mux_m7,mux_m8
.break8		+mux_writer_link

		+mux_writer mux_x9,mux_y9,mux_p9,mux_c9,mux_l9
		+mux_writer_msbx 1,mux_m8,mux_m9
.break9		+mux_writer_link

		+mux_writer mux_x10,mux_y10,mux_p10,mux_c10,mux_l10
		+mux_writer_msbx 2,mux_m9,mux_m10
.break10	+mux_writer_link

		+mux_writer mux_x11,mux_y11,mux_p11,mux_c11,mux_l11
		+mux_writer_msbx 3,mux_m10,mux_m11
.break11	+mux_writer_link

		+mux_writer mux_x12,mux_y12,mux_p12,mux_c12,mux_l12
		+mux_writer_msbx 4,mux_m11,mux_m12
.break12	+mux_writer_link

		+mux_writer mux_x13,mux_y13,mux_p13,mux_c13,mux_l13
		+mux_writer_msbx 5,mux_m12,mux_m13
.break13	+mux_writer_link

		+mux_writer mux_x14,mux_y14,mux_p14,mux_c14,mux_l14
		+mux_writer_msbx 6,mux_m13,mux_m14
.break14	+mux_writer_link

		+mux_writer mux_x15,mux_y15,mux_p15,mux_c15,mux_l15
		+mux_writer_msbx 7,mux_m14,mux_m15
.break15	+mux_writer_link


		;;;;;;;; 16-23 ;;;;;;;;

		+mux_writer mux_x16,mux_y16,mux_p16,mux_c16,mux_l16
		+mux_writer_msbx 0,mux_m15,mux_m16
.break16	+mux_writer_link

		+mux_writer mux_x17,mux_y17,mux_p17,mux_c17,mux_l17
		+mux_writer_msbx 1,mux_m16,mux_m17
.break17	+mux_writer_link

		+mux_writer mux_x18,mux_y18,mux_p18,mux_c18,mux_l18
		+mux_writer_msbx 2,mux_m17,mux_m18
.break18	+mux_writer_link

		+mux_writer mux_x19,mux_y19,mux_p19,mux_c19,mux_l19
		+mux_writer_msbx 3,mux_m18,mux_m19
.break19	+mux_writer_link

		+mux_writer mux_x20,mux_y20,mux_p20,mux_c20,mux_l20
		+mux_writer_msbx 4,mux_m19,mux_m20
.break20	+mux_writer_link

		+mux_writer mux_x21,mux_y21,mux_p21,mux_c21,mux_l21
		+mux_writer_msbx 5,mux_m20,mux_m21
.break21	+mux_writer_link

		+mux_writer mux_x22,mux_y22,mux_p22,mux_c22,mux_l22
		+mux_writer_msbx 6,mux_m21,mux_m22
.break22	+mux_writer_link

		+mux_writer mux_x23,mux_y23,mux_p23,mux_c23,mux_l23
		+mux_writer_msbx 7,mux_m22,mux_m23
.break23	+mux_writer_link


		;;;;;;;; 24-31 ;;;;;;;;

		+mux_writer mux_x24,mux_y24,mux_p24,mux_c24,0
		+mux_writer_msbx 0,mux_m23,mux_m24
.break24	+mux_writer_link

		+mux_writer mux_x25,mux_y25,mux_p25,mux_c25,0
		+mux_writer_msbx 1,mux_m24,mux_m25
.break25	+mux_writer_link

		+mux_writer mux_x26,mux_y26,mux_p26,mux_c26,0
		+mux_writer_msbx 2,mux_m25,mux_m26
.break26	+mux_writer_link

		+mux_writer mux_x27,mux_y27,mux_p27,mux_c27,0
		+mux_writer_msbx 3,mux_m26,mux_m27
.break27	+mux_writer_link

		+mux_writer mux_x28,mux_y28,mux_p28,mux_c28,0
		+mux_writer_msbx 4,mux_m27,mux_m28
.break28	+mux_writer_link

		+mux_writer mux_x29,mux_y29,mux_p29,mux_c29,0
		+mux_writer_msbx 5,mux_m28,mux_m29
.break29	+mux_writer_link 

		+mux_writer mux_x30,mux_y30,mux_p30,mux_c30,0
		+mux_writer_msbx 6,mux_m29,mux_m30
.break30	+mux_writer_link 

		+mux_writer mux_x31,mux_y31,mux_p31,mux_c31,0
		+mux_writer_msbx 7,mux_m30,mux_m31
		rts