top_lines	= 38
top_offset	= 63*top_lines
top_line	= $2d-top_lines
bottom_line	= $f7


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!zone
top_irq		sta irq_save_a


		;;;;;;;; switch C128s back to slow mode ;;;;;;;;

		lda #%00000000
		sta vic_clock


		;;;;;;;; stabilize interrupt ;;;;;;;;

		lda #<.stable_irq	;; use the double-interrupt method
		sta cpu_irq+0
		lda #>.stable_irq
		sta cpu_irq+1
		inc vic_line
		lsr vic_irq
		cli

		!for i,30 { nop }	;; -.
					;;  |- fill this with useful code
.stable_irq	+delay 63-17		;; -'

		lda vic_line
		cmp vic_line
		beq *+2

		pla
		pla
		pla


		;;;;;;;; program the cia timers ;;;;;;;;

		lda #%01010001		;; star the timers
		sta cia1_cra
		sta cia1_crb

cia_model	= *+1			;; adjust for older cia models which
		lda #$00		;; trigger interrupts one cycle late.
		bne *+2			;; be careful only to apply it to the
					;; interrupt sources though

		lda #%00010001
		sta cia2_cra
		sta cia2_crb

		lda #63*2-1		;; set timer periods
		sta cia1_ta_lo
		lda #8-1
		sta cia1_tb_lo

		lda #<63*16-1
		sta cia2_ta_lo
		lda #>63*16-1
		sta cia2_ta_hi

		lda #<63*8-1
		sta cia2_tb_lo
		lda #>63*8-1
		sta cia2_tb_hi

		lda #%10000011		;; enable interrupts
		sta cia2_icr


		;;;;;;;; start the multiplexer's irq chain ;;;;;;;;

	;;	inc vic_cborder		;; flash the border a bit to prove
	;;	dec vic_cborder		;; that we're stable

		jsr mux_display

		lda irq_save_a
		lsr vic_irq
		rti


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
		!zone

bottom_irq	!zone
		pha			;; must be reentrant
		txa
		pha
		tya
		pha

		cld


		;;;;;;;; timer configuration ;;;;;;;;

		lda #%00010000		;; stop the timers
		sta cia2_cra
		sta cia2_crb
		sta cia1_cra
		sta cia1_crb

		lda #%00000000		;; hide the sprites so they don't
		sta vic_sactive		;; steal any cycles in the border

		bit cia2_icr		;; acknowledge restore NMIs

		lda #63*2+21		;; latch new base values for the counters
		sta cia1_ta_lo
		lda #(top_lines>>1)&7
		sta cia1_tb_lo

		lda #<top_offset+63*8-9
		sta cia2_ta_lo
		lda #>top_offset+63*8-9
		sta cia2_ta_hi

		!if panel_ylock >= 0 {
		lda #panel_ylock
		} else {
		lda view_ypos
		}
		and #%1000
		beq .early

.late		lda #<top_offset+63*4-12
		sta cia2_tb_lo
		lda #>top_offset+63*4-12
		sta cia2_tb_hi
		jmp .continue

.early		lda #<top_offset+63*4-12 - 63*8
		sta cia2_tb_lo
		lda #>top_offset+63*4-12 - 63*8
		sta cia2_tb_hi

		lda #<dc_early_nmi
		sta cpu_nmi+0

.continue

		;;;;;;;; time to die? ;;;;;;;;

		lda #%00000001
raster_active	bne .still_alive
raster_active_run = opc_bne_rel
raster_active_break = opc_beq_rel

		lda #game_active_break
		sta game_active

		lsr
	;;	lda #%00000000
		sta vic_irqmask

		lsr vic_irq
		jmp .abort


		;;;;;;;; switch C128s to fast mode ;;;;;;;;

.still_alive	sta vic_clock


		;;;;;;;; reset vic registers ;;;;;;;;

		!if panel_ylock >= 0 {
		lda #panel_ylock
		} else {
		lda view_ypos
		}
		lsr
		ora #%00000011
		eor #%00000100
		sta vic_ram

		lda #%00010111
		sta vic_controlh

		!if panel_ylock >= 0 {
		lda #panel_ylock
		} else {
		lda view_ypos
		}
		and #%00000111
		ora #%00010000
		sta vic_controlv


		;;;;;;;; trigger next irq ;;;;;;;;

		+panel_prepare

		!if panel_ylock >= 0 {
		lda #panel_ylock
		} else {
		lda view_ypos
		}
		eor #%1000
		clc
		adc #top_line
		sta vic_line

		lda #<top_irq
		sta cpu_irq+0
		lda #>top_irq
		sta cpu_irq+1
		lsr vic_irq

		cli			;; beyond this line the interrupt
					;; is reentrant!


		;;;;;;;; run the game logic ;;;;;;;;

		inc game_ready
		bne .locked

		lda actor_count
		asl
		asl
		adc actor_count
		tax
		sbx #8*mux_cmask_pitch	;; pitch = 5
		bcs .dont_clip
		ldx #0
.dont_clip	lda vic_ss_collided
		sta mux_cmask,x

		jsr collision_cache_masks

	;;	inc vic_cborder
		jsr mux_write
	;;	dec vic_cborder

	;;	dec vic_cborder
		jsr game_logic
	;;	inc vic_cborder

.locked		dec game_ready

.abort		pla
		tay
		pla
		tax
		pla
		rti