#include <stdio.h>

typedef struct {
	unsigned char b, g, r;
} rgb_t;

// define the VIC palette used
static const rgb_t vic_palette[] = {
#	define BGR(r, g, b) { b, g, r }

#	if 0
	// GangEd
	BGR(0x00, 0x00, 0x00),
	BGR(0xff, 0xff, 0xff),
	BGR(0x89, 0x40, 0x36),
	BGR(0x7a, 0xbf, 0xc7),
	BGR(0x8a, 0x46, 0xae),
	BGR(0x68, 0xa9, 0x41),
	BGR(0x3e, 0x31, 0xa2),
	BGR(0xd0, 0xdc, 0x71),
	BGR(0x90, 0x5f, 0x25),
	BGR(0x5c, 0x47, 0x00),
	BGR(0xbb, 0x77, 0x6d),
	BGR(0x55, 0x55, 0x55),
	BGR(0x80, 0x80, 0x80),
	BGR(0xac, 0xea, 0x88),
	BGR(0x7c, 0x70, 0xda),
	BGR(0xab, 0xab, 0xab),
#	elif 0
	// Deekay
	BGR(0x00, 0x00, 0x00),
	BGR(0xff, 0xff, 0xff),
	BGR(0x68, 0x37, 0x2b),
	BGR(0x70, 0xa4, 0xb2),
	BGR(0x6f, 0x3d, 0x86),
	BGR(0x58, 0x8d, 0x43),
	BGR(0x35, 0x28, 0x79),
	BGR(0xb8, 0xc7, 0x6f),
	BGR(0x6f, 0x4f, 0x25),
	BGR(0x43, 0x39, 0x00),
	BGR(0x9a, 0x67, 0x59),
	BGR(0x44, 0x44, 0x44),
	BGR(0x6c, 0x6c, 0x6c),
	BGR(0x9a, 0xd2, 0x84),
	BGR(0x6c, 0x5e, 0xb5),
	BGR(0x95, 0x95, 0x95)
#	else
	// VICE
	BGR(0x00, 0x00, 0x00),
	BGR(0xfd, 0xfe, 0xfc),
	BGR(0xbe, 0x1a, 0x24),
	BGR(0x30, 0xe6, 0xc6),
	BGR(0xb4, 0x1a, 0xe2),
	BGR(0x1f, 0xd2, 0x1e),
	BGR(0x21, 0x1b, 0xae),
	BGR(0xdf, 0xf6, 0x0a),
	BGR(0xb8, 0x41, 0x04),
	BGR(0x6a, 0x33, 0x04),
	BGR(0xfe, 0x4a, 0x57),
	BGR(0x42, 0x45, 0x40),
	BGR(0x70, 0x74, 0x6f),
	BGR(0x59, 0xfe, 0x59),
	BGR(0x5f, 0x53, 0xfe),
	BGR(0xa4, 0xa7, 0xa2)
#	endif
};

// search for the nearest color in the (partial) palette 
static unsigned nearest(rgb_t color, unsigned limit) {
	unsigned index;
	unsigned best_index, best_distance = -1;

	for(index = 0; index < limit; ++index) {
		unsigned distance;

		unsigned r = vic_palette[index].r - color.r;
		unsigned g = vic_palette[index].g - color.g;
		unsigned b = vic_palette[index].b - color.b;

		r *= 30;
		g *= 59;
		b *= 11;

		r *= r;
		g *= g;
		b *= b;

		distance = r + g + b;

		if(distance <= best_distance)
		{
			best_index = index;
			best_distance = distance;
		}
	}

	return best_index;
}

// interpolate between two colors
enum { NUM_FRAMES = 8 };

static rgb_t interpolate(rgb_t src, signed frame, rgb_t dst)
{
	src.r += (dst.r - src.r) * frame / NUM_FRAMES;
	src.g += (dst.g - src.g) * frame / NUM_FRAMES;
	src.b += (dst.b - src.b) * frame / NUM_FRAMES;

	return src;
}

// generate the fade tables
int main(int argc, char *argv[]) {
	unsigned frame;
	unsigned index;

	rgb_t source = vic_palette[0];

	if(argc >= 2) {
		freopen(argv[1], "wb", stdout);

		if(argc >= 3) {
			source = vic_palette[atoi(argv[2]) & 15];
		}
	}

	// 16-color vic register
	for(frame = 1; frame <= NUM_FRAMES; ++frame) {
		for(index = 0; index < 16; ++index) {
			rgb_t color = interpolate(source, frame, vic_palette[index]);

			fputc(nearest(color, 16), stdout);
		}
	}

	// 8-color color ram
	for(frame = 1; frame <= NUM_FRAMES; ++frame) {
		for(index = 0; index < 16; ++index) {
			rgb_t color = interpolate(source, frame, vic_palette[index & 7]);

			fputc(nearest(color, 8) + (index & 8), stdout);
		}
	}

	return 0;
}